/*
 * Usage: boot bootclient bootserver
 *
 * Executes a bootparam WHOAMI request and print the results.
 */

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <rpc/rpc.h>
#include <netdb.h>
#include <tcp.h>
#include "bootpara.h"

int main (int argc, char **argv)
{
  int    stat;
  struct hostent *hp;
  static struct bp_whoami_arg bp_arg;
  static struct bp_whoami_res bp_res;
  char  *me     = argv[0];
  char  *client = argv[1];
  char  *server = argv[2];
  u_long addr;

  if (argc != 3)
  {
    fprintf (stderr, "Usage: %s bootclient bootserver\n", me);
    return (1);
  }
  dbug_init();
  rpc_init();
  sock_init();

  /* Find out the bootserver's official host name.
   */
  if ((hp = gethostbyname(server)) == NULL)
  {
    fprintf (stderr, "Host %s not found.\n", server);
    return (1);
  }
  /* Find out the bootclient's address. Assume it has only one.
   */
  if ((hp = gethostbyname(client)) != NULL)
          memcpy (&bp_arg.client_address.bp_address_u.ip_addr,
                  hp->h_addr_list[0], hp->h_length);
  else if ((addr = inet_addr(client)) != INADDR_NONE)
          *(u_long*) &bp_arg.client_address.bp_address_u.ip_addr = addr;
  else
  {
    fprintf (stderr, "Host %s not found.\n", client);
    return (1);
  }
  bp_arg.client_address.address_type = IP_ADDR_TYPE;
  bp_res.client_name = 0;                     /* allocate buffer */
  bp_res.domain_name = 0;                     /* allocate buffer */

  stat = callrpc (server, BOOTPARAMPROG, BOOTPARAMVERS,
                  BOOTPARAMPROC_WHOAMI,
                  xdr_bp_whoami_arg, (char*)&bp_arg,
                  xdr_bp_whoami_res, (char*)&bp_res);
  if (stat)
  {
    fprintf (stderr, "Cannot contact bootparam server at `%s' for `%s':\n%s\n",
             server, client, clnt_sperrno(stat));
    return (1);
  }
  printf ("client_name: %s\n", bp_res.client_name);
  printf ("domain_name: %s\n", bp_res.domain_name);
  printf ("router_addr: %d.%d.%d.%d\n",
          bp_res.router_address.bp_address_u.ip_addr.net & 255,
          bp_res.router_address.bp_address_u.ip_addr.host & 255,
          bp_res.router_address.bp_address_u.ip_addr.lh & 255,
          bp_res.router_address.bp_address_u.ip_addr.impno & 255);
  return (0);
}
