/*
 * secure identity verifier and reporter: client side
 */
#include <stdio.h>
#include <rpc/rpc.h>
#include <tcp.h>
#include "whoami.h"

/*
 * Before running this program, the user must have a key in the publickey
 * database, and must have logged in with a password (or used keylogin).
 * The user's machine and the server's machine must both be running keyserv.
 */

int main (int argc, char **argv)
{
  CLIENT          *cl;
  char            *server;
  remote_identity *remote_me;
  name            *servername;
  void            *nullp;

  if (argc != 2)
  {
    fprintf (stderr, "usage: %s host\n", argv[0]);
    return (1);
  }

#ifdef DEBUG
  dbug_init();
#endif
  rpc_init();
  sock_init();

  server = argv[1];

  /*
   * Create client "handle" used for calling WHOAMI on the
   * server designated on the command line. We tell the rpc package
   * to use the "udp" protocol when contacting the server.
   */
  cl = clnt_create (server, WHOAMI, WHOAMI_V1, "udp");
  if (!cl)
  {
    /*
     * Couldn't establish connection with server.
     * Print error message and die.
     */
    clnt_pcreateerror (server);
    return (1);
  }
  /*
   * Get network identifier for server machine.
   */
  servername = whoami_whoru_1 (nullp, cl);
  if (!servername)
  {
    fprintf (stderr, "Trouble communicating with %s\n",
             clnt_sperror(cl,server));
    return (1);
  }
  if (*servername[0] == '\0')
  {
    fprintf (stderr, "Could not determine netname of WHOAMI server.\n");
    return (1);
  }
  printf ("Server's netname is: %s\n", *servername);

  /*
   * A wide window and no synchronization is used.  Client and server
   * clock must be with five minutes of each other.
   */
  cl->cl_auth = authdes_create (*servername, 60*5, NULL, NULL);

  /*
   *  Find out who I am, in the server's point of view.
   */
  remote_me = whoami_iask_1 (nullp, cl);
  if (!remote_me)
  {
    fprintf (stderr, "Trouble getting my identity from %s\n",
             clnt_sperror(cl, server));
    return (1);
  }
  /*
   * Print out my identity.
   */
  printf ("My remote user name: %s\n", remote_me->remote_username);
  printf ("My remote real name: %s\n", remote_me->remote_realname);

  return(0);
}
