/*
 * client program for remote directory listing client
 */

#include <stdio.h>
#include <stdio.h>
#include <errno.h>
#include <rpc/rpc.h>   /* always need this */
#include <tcp.h>
#include "dir.h"       /* need this too: will be generated by rpcgen*/

int main (int argc, char **argv)
{
  CLIENT      *cl;
  char        *server;
  char        *dir;
  readdir_res *result;
  namelist nl;

  if (argc != 3)
  {
    fprintf (stderr, "usage: %s host directory\n", argv[0]);
    return (1);
  }

#ifdef DEBUG
  dbug_init();
#endif
  rpc_init();
  sock_init();

  /*
   * Remember what our command line arguments refer to
   */
  server = argv[1];
  dir = argv[2];

  /*
   * Create client "handle" used for calling DIRPROG on the
   * server designated on the command line. We tell the rpc package
   * to use the "tcp" protocol when contacting the server.
   */
  cl = clnt_create (server, DIRPROG, DIRVERS, "tcp");
  if (cl == NULL)
  {
    /*
     * Couldn't establish connection with server.
     * Print error message and die.
     */
    clnt_pcreateerror (server);
    return (1);
  }
  
  /*
   * Call the remote procedure "readdir" on the server
   */
  result = readdir_1(&dir, cl);
  if (result == NULL)
  {
    /*
     * An error occurred while calling the server. 
     * Print error message and die.
     */
    clnt_perror (cl, server);
    return (1);
  }

  /*
   * Okay, we successfully called the remote procedure.
   */
  if (result->errno)
  {
    /*
     * A remote system error occurred.
     * Print error message and die.
     */
    errno = result->errno;
    perror (dir);
    return (1);
  }

  /*
   * Successfuly got a directory listing.
   * Print it out.
   */
  for (nl = result->readdir_res_u.list; nl; nl = nl->next)
      printf ("%s\n", nl->name);

}
