/*
 * RE_SID: @(%)/tmp_mnt/vol/dosnfs/shades_SCCS/unix/pcnfsd/v2/src/SCCS/
 * s.pcnfsd_misc.c 1.13 93/02/08 10:18:54 SMI
 */

/*
**=====================================================================
** Copyright (c) 1986-1993 by Sun Microsystems, Inc.
**	@(#)pcnfsd_misc.c	1.13	2/8/93
**=====================================================================
*/

#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <pwd.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <unistd.h>
#include <process.h>
#include <rpc/rpc.h>

#include "common.h"
#include "pcnfsd.h"
#include "extern.h"

#define READER_FD  0
#define WRITER_FD  1

char tempstr[256];

int  interrupted = 0;
int  uidrsize    = 0;
int  uidrlo[NUMUIDS];
int  uidrhi[NUMUIDS];

static int    child_pid;
static char   cached_user[64] = "";
static uid_t  cached_uid;
static gid_t  cached_gid;
static struct itimerval timer;
static FILE  *pipe_handle;

/* 
 * bulletproof malloc
 */
void *grab (int n)
{
  void *p = malloc (n);

  if (!p)
  {
    msg_out ("rpc.pcnfsd: malloc failure");
    exit (1);
  }
  return (p);
}

char *my_strdup (char *s)
{
  char *r = (char*) grab (strlen(s)+1);
  strcpy (r, s);
  return (r);
}

/*
 * This is the latest word on the security check. The following
 * routine "suspicious()" returns non-zero if the character string
 * passed to it contains any shell meta-characters.
 * Callers will typically code
 *
 *  if (suspicious(some_parameter)) reject();
 *
 * Non-Unix-based systems: examine this carefully.
 */
int suspicious (char *s)
{
  if (strpbrk(s,";|&<>`'#!?*()[]^/"))
     return (1);
  return (0);
}

void scramble (char *s1, char *s2)
{
  while (*s1)
  {
    *s2++ = (*s1 ^ zchar) & 0x7f;
    s1++;
  }
  *s2 = 0;
}


/* 
 * build_uid_ranges(s)
 */
static void build_uid_ranges (char *s)
{
  char *r1, *r2;
  int   v1,  v2;

  r1 = strtok (s, " ,\t");
  while (r1 && uidrsize < NUMUIDS)
  {
    if ((r2 = strchr (r1, '-')) != NULL)
      *r2++ = '\0';

    v1 = atoi (r1);
    if (v1 == 0 && *r1 != '0')	/* syntax error */
      continue;
    if (r2)
    {
      v2 = atoi (r2);
      if (v2 == 0 && *r2 != 0)	/* syntax */
	continue;
      if (v2 < v1)		/* botch */
	continue;
    }
    else
      v2 = v1;
    uidrlo[uidrsize] = v1;
    uidrhi[uidrsize] = v2;
    uidrsize++;
    r1 = strtok (NULL, " ,\t");
  }
}

/*
 * val_uid(u)
 *
 * Return non-zero if the uid is legal - within range
 * and zero otherwise.
 *
 * If uid_range is NULL, check for 101-60002. Otherwise
 * check against ranges.
 */
static int val_uid (int u)
{
  int i;

  if (uidrsize == 0)
     return (u >= 101 && u <= 60002);

  for (i = 0; i < uidrsize; i++)
    if (u >= uidrlo[i] && u <= uidrhi[i])
      return (1);
  return (0);
}


struct passwd *get_password (char *usrnam, char **msgpp)
{
  static struct passwd localp;
  struct passwd *p = getpwnam (usrnam);
  char  *pswd;
  char  *ushell;

  if (!p)
    return (NULL);

#ifdef __MSDOS__
  localp = *p;
#else
  pswd   = p->pw_passwd;
  localp = *p;
  localp.pw_passwd = pswd;
#endif

 /*
  * Fix SDR#pcn1857: If the shell field is null, this entry is legal
  * (assuming that the default shell, "/bin/sh" is always legal).
  */
  if (localp.pw_shell == NULL || strlen(localp.pw_shell) == 0)
     return (&localp);

#ifdef USE_GETUSERSHELL
  {
    int ok = 0;
    setusershell();
    while ((ushell = getusershell()) != NULL)
    {
      if (!strcmp (ushell, localp.pw_shell))
      {
        ok = 1;
        break;
      }
    }
    endusershell();
    if (!ok)
       return (NULL);
  }
#else
 /*
  * the best we can do is to ensure that the shell ends in "sh"
  */
  ushell = localp.pw_shell;
  if (strlen(ushell) < 2)
     return (NULL);

  ushell += strlen (ushell) - 2;
  if (strcmp (ushell, "sh"))
     return (NULL);
#endif

  if (!val_uid(localp.pw_uid))
  {
    if (msgpp)
       *msgpp = "No access via pcnfsd to this account.";
    return (NULL);
  }
  return (&localp);
}


char *mapfont (char f, char i, char b)
{
  static char fontname[64];

  fontname[0] = 0;		/* clear it out */

  switch (f)
  {
    case 'c':
	 strcpy (fontname, "Courier");
	 break;
    case 'h':
	 strcpy (fontname, "Helvetica");
	 break;
    case 't':
	 strcpy (fontname, "Times");
	 break;
    default:
	 strcpy (fontname, "Times-Roman");
	 goto finis;
  }
  if (i != 'o' && b != 'b')
  {				/* no bold or oblique */
    if (f == 't')		/* special case Times */
       strcat (fontname, "-Roman");
    goto finis;
  }
  strcat (fontname, "-");
  if (b == 'b')
     strcat (fontname, "Bold");
  if (i == 'o')			/* o-blique */
     strcat (fontname, f == 't' ? "Italic" : "Oblique");

finis:
  return (&fontname[0]);
}

/* 
 * run_ps630 performs the Diablo 630 emulation filtering process. ps630
 * was broken in certain Sun releases: it would not accept point size or
 * font changes.
 */
void run_ps630 (char *f, char *opts)
{
  char temp_file[256];
  char commbuf[256];
  int i;

  strcpy (temp_file, f);
  strcat (temp_file, "X");       /* intermediate file name */

  /*
   * The pitch and font features of ps630 appear to be broken at
   * this time.
   */
  sprintf (commbuf, "ps630 -p %s %s", temp_file, f);

  if ((i = system(commbuf)) != 0)
  {
    /*
     * Under (un)certain conditions, ps630 may return -1 even
     * if it worked. Hence the commenting out of this error
     * report.
     */
#if 0
    fprintf (stderr, "\n\nrun_ps630 rc = %d\n", i);
    exit (1);
#endif
  }
  if (rename(temp_file, f))
  {
    perror ("run_ps630: rename");
    exit (1);
  }
}



static void myhandler (void)
{
  interrupted = 1;
  fclose (pipe_handle);
  kill (child_pid, SIGKILL);
  msg_out ("rpc.pcnfsd: su_popen timeout - killed child process");
}

static void start_watchdog (int n)
{
  /* setup SIGALRM handler, force interrupt of ongoing syscall
   */
  signal (SIGALRM, (void(*)(int))myhandler);

  /* set interval timer for n seconds
   */
  timer.it_interval.tv_sec  = 0;
  timer.it_interval.tv_usec = 0;
  timer.it_value.tv_sec  = n;
  timer.it_value.tv_usec = 0;
  setitimer (ITIMER_REAL, &timer, NULL);
  interrupted = 0;

}

static void stop_watchdog (void)
{
  /* cancel timer
   */
  timer.it_interval.tv_sec  = 0;
  timer.it_interval.tv_usec = 0;
  timer.it_value.tv_sec  = 0;
  timer.it_value.tv_usec = 0;
  setitimer (ITIMER_REAL, &timer, NULL);

  /* restore old signal handling
   */
  signal (SIGALRM, SIG_IGN);
}


FILE *su_popen (char *user, char *cmd, int maxtime)
{
  struct passwd *pw;
  int    p[2], parent_fd, child_fd, pid;
  char   tmp[256];

  if (strcmp(cached_user, user))
  {
    pw = getpwnam (user);
    if (!pw)
       pw = getpwnam ("nobody");
    if (pw)
    {
      cached_uid = pw->pw_uid;
      cached_gid = pw->pw_gid;
      strcpy (cached_user, user);
    }
    else
    {
      cached_uid = NOBODY;
      cached_gid = (gid_t) NOBODY;
      cached_user[0] = '\0';
    }
  }
  if (cached_uid != NOBODY && !val_uid((int)cached_uid))
  {
    sprintf (tmp, "rpc.pcnfsd: rejecting request from user %s (uid %d)",
             user, (int)cached_uid);
    msg_out (tmp);
    return (NULL);
  }

  if (pipe(p) < 0)
  {
    msg_out ("rpc.pcnfsd: unable to create pipe in su_popen");
    return (NULL);
  }

  parent_fd = p [READER_FD];
  child_fd  = p [WRITER_FD];

  if ((pid = fork()) == 0)
  {
    int i;

    for (i = 0; i < 10; i++)
      if (i != child_fd)
         close (i);
    if (child_fd != 1)
    {
      dup2 (child_fd, 1);
      close (child_fd);
    }
    dup2 (1, 2);		/* let's get stderr as well */

    setgid (cached_gid);
    setuid (cached_uid);
    execl ("/bin/sh", "sh", "-c",cmd,NULL);
    _exit (255);
  }
  if (pid == -1)
  {
    msg_out ("rpc.pcnfsd: fork failed");
    close (parent_fd);
    close (child_fd);
    return (NULL);
  }
  child_pid = pid;
  close (child_fd);
  start_watchdog (maxtime);
  pipe_handle = fdopen (parent_fd, "r");
  return (pipe_handle);
}

int su_pclose (FILE *ptr)
{
  int pid, status;

  stop_watchdog();

  fclose (ptr);
  if (child_pid == -1)
     return (-1);

  while ((pid = wait(&status)) != child_pid && pid != -1)
       ;
  return (pid == -1 ? -1 : status);
}


/*
 * The following routine reads a file "/etc/pcnfsd.conf" if present,
 * and uses it to replace certain builtin elements, like the
 * name of the print spool directory. The configuration file
 * Is the usual kind: Comments begin with '#', blank lines are ignored,
 * and valid lines are of the form
 *
 *      <keyword><whitespace><value>
 *
 * The following keywords are recognized:
 *
 *      spooldir dirname
 *      printer name alias-for command
 *      wtmp yes|no
 */
void config_from_file (void)
{
  FILE *fd;
  char *cp, *kw, *val;
  char *arg1, *arg2;
  char buff[1024];

  if ((fd = fopen ("/etc/pcnfsd.conf", "r")) == NULL)
     return;

  while (fgets (buff,sizeof(buff),fd))
  {
    cp  = strchr (buff, '\n');
    *cp = '\0';
    cp  = strchr (buff, '#');
    if (cp)
       *cp = '\0';

    kw = strtok (buff, " \t");
    if (kw == NULL)
       continue;

    val = strtok (NULL, " \t");
    if (!val)
       continue;

    if (!stricmp(kw, "spooldir"))
    {
      strcpy (sp_name, val);
      continue;
    }
    if (!stricmp(kw, "uidrange"))
    {
      build_uid_ranges (val);
      continue;
    }
    if (!stricmp(kw, "printer"))
    {
      arg1 = strtok (NULL, " \t");
      arg2 = (arg1 == NULL ? NULL : strtok (NULL, ""));
      add_printer_alias (val, arg1, arg2);
      continue;
    }
   /*
    * Add new cases here
    */
  }
  fclose (fd);
}

/*
 *
 * strembedded - returns true if s1 is embedded (in any case) in s2
 */
int strembedded (char *s1, char *s2)
{
  while (*s2)
  {
    if (!stricmp(s1, s2))
       return (1);
    s2++;
  }
  return (0);
}
