/* @(#)rpc_main.c  2.2 88/08/01 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

/*
 * rpc_main.c, Top level of the RPC protocol compiler. 
 * Copyright (C) 1987, Sun Microsystems, Inc. 
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <process.h>
#include <tcp.h>

#include "rpc_scan.h"
#include "rpc_pars.h"
#include "rpc_util.h"

struct commandline cmd;

static int  allfiles;

static char *cmdname = "rpcgen";

static char *allv[] = {
            "rpcgen", "-s", "udp", "-s", "tcp",
          };
static int allc = sizeof(allv)/sizeof(allv[0]);

/*
 * Prototypes
 */

static void do_regs   (int argc, char **argv);
static void parseargs (int, char**);
static void c_output  (char*, char*, int, char*);
static void h_output  (char*, char*, int, char*);
static void l_output  (char*, char*, int, char*);
static void s_output  (int, char**, char*, char*, int, char*, int);
static void usage     (char *str);

int main (int argc, char **argv)
{
  parseargs (argc, argv);

  if (cmd.cflag)
  {
    c_output (cmd.infile, "-DRPC_XDR", 0, cmd.outfile);
    return (0);
  }
  if (cmd.hflag)
  {
    h_output (cmd.infile, "-DRPC_HDR", 0, cmd.outfile);
    return (0);
  }
  if (cmd.lflag)
  {
    l_output (cmd.infile, "-DRPC_CLNT", 0, cmd.outfile);
    return (0);
  }
  if (cmd.sflag || cmd.mflag)
  {
    s_output (argc, argv, cmd.infile, "-DRPC_SVC", 0, cmd.outfile, cmd.mflag);
    return (0);
  }

  c_output (cmd.infile, "-DRPC_XDR", 1, "_x.c");
  reinitialize();
  h_output (cmd.infile, "-DRPC_HDR", 1, ".h");
  reinitialize();
  l_output (cmd.infile, "-DRPC_CLNT", 1, "_c.c");
  reinitialize();
  s_output (allc, allv, cmd.infile, "-DRPC_SVC", 1, "_s.c", cmd.mflag);

  return (0);
}

static void usage (char *str)
{
  if (str)
     fprintf (stderr, "%s\n", str);
  fprintf (stderr,
           "usage: %s [options] [-o outfile] infile\n"
           "options: [-vk] [-c | -h | -l | -m | -s {udp,tcp}]\n"
           "\t -v      shows verbose information\n"
           "\t -k      keep temporary files\n"
           "\t -h      generate common header (.h) file\n"
           "\t -c      generate XDR stub file\n"
           "\t -l      generate client source (.c) file\n"
           "\t -m      don't write main() to server file\n"
           "\t -s udp  generate UDP server source (.c) file\n"
           "\t -s tcp  generate TCP server source (.c) file\n",
           cmdname);
  exit (1);
}

/*
 * add extension to filename 
 */
static char *extendfile (char *file, char *ext)
{
  char *res = malloc (strlen(file) + strlen(ext) + 1);
  char *p   = strrchr (file, '.');

  if (!p)
     p = strchr (file, '\0');

  strcpy (res, file);
  strcpy (res + (p - file), ext);
  return (res);
}

/*
 * Open output file with given extension 
 */
static void open_output (char *infile, char *outfile)
{
  time_t now = time (NULL);

  if (fout)
  {
    fclose (fout);
    fout = NULL;
  }

  if (outfile == NULL)
     fout = stdout;
  else
  {
    if (infile && streq(outfile,infile))
    {
      fprintf (stderr, "output would overwrite %s\n", infile);
      crash();
    }
    fout = fopen (outfile, WRITE_MODE);
    if (!fout)
    {
      fprintf (stderr, "unable to open `%s'", outfile);
      crash();
    }
  }
  fprintf (fout,
           "/*\n"
           " * This is a RPCGEN generated file. Do not edit.\n"
           " *\n"
           " * Generated: %s"
           " */\n\n", ctime(&now));

  record_open (outfile);
}

static char *cpp_out = "rpcgen.$$$";

static void remove_temp (void)
{
  if (!cmd.keep)
     unlink (cpp_out);
}

/*
 * Open input file with given define for C-preprocessor 
 */
static void open_input (char *infile, char *define)
{

  char *cpp     = getenv ("RPCGEN_CPP");
  char  cpp_cmd[512];

  if (!infile)
     infile = "";
  if (!define)
     define = "";
  if (fin)
  {
    fclose (fin);
    fin = NULL;
  }
  if (!cpp)
  {
    fprintf (stderr,"Environment variable `RPCGEN_CPP' not set.\n");
    fprintf (stderr,"E.g. `RPCGEN_CPP=c:\\djgpp\\bin\\cpp.exe'\n");
    crash();
  }
  sprintf (cpp_cmd, "%s %s -o %s %s", cpp, define, cpp_out, infile);
  VERBOSE1 ("cpp: %s\n", cpp_cmd);

  if (system(cpp_cmd))
  {
    fprintf (stderr, "exec of `cpp' failed!\n");
    crash();
  }
  atexit (remove_temp);

  fin = fopen (cpp_out, READ_MODE);
  if (!fin)
  {
    fprintf (stderr, "cannot open cpp output `%s'\n",cpp_out);
    crash();
  }
}

static void put_headers (char *infile)
{
  char *include;

  fprintf (fout,
           "#include <stdio.h>\n"
           "#include <string.h>\n"
           "#include <rpc/rpc.h>\n");

  if (infile && (include = extendfile(infile,".h")) != NULL)
  {
    fprintf (fout, "#include \"%s\"\n", include);
    free (include);
  }
}

/*
 * Compile into an XDR routine output file
 */
static void c_output (char *infile, char *define, int extend, char *outfile)
{
  definition *def;
  char       *outfilename = outfile;
  long        tell;

  open_input (infile, define);
  if (extend)
     outfilename = extendfile (infile, outfile);
  open_output (infile, outfilename);
  put_headers (infile);

  tell = ftell (fout);
  while ((def = get_definition()) != NULL)
       emit (def);

  if (extend)
  {
    if (tell == ftell(fout) && !cmd.keep)
       unlink (outfilename);
    free (outfilename);
  }
}

/*
 * Compile into an XDR header file
 */
static void h_output (char *infile, char *define, int extend, char *outfile)
{
  definition *def;
  char       *outfilename = outfile;
  char       *h_def;
  long        tell;

  open_input (infile, define);
  if (extend)
     outfilename = extendfile (infile, outfile);
  open_output (infile, outfilename);

  h_def = strupr (extendfile(infile,"_H"));

  fprintf (fout, "#ifndef __%s\n#define __%s\n\n", h_def, h_def);

  tell = ftell (fout);

  while ((def = get_definition()) != NULL)
        print_datadef (def);

  fprintf (fout, "#endif /* !__%s */\n", h_def);
  free (h_def);

  if (extend)
  {
    if (tell == ftell(fout) && !cmd.keep)
       unlink (outfilename);
    free (outfilename);
  }
}

/*
 * Compile into an RPC service
 */
static void s_output (int argc, char **argv, char *infile, char *define,
                      int extend, char *outfile, int nomain)
{
  definition *def;
  int         foundprogram;
  char       *outfilename;

  open_input (infile, define);
  outfilename = extend ? extendfile(infile, outfile) : outfile;
  open_output (infile, outfilename);
  put_headers (infile);

  foundprogram = 0;
  while ((def = get_definition()) != NULL)
        foundprogram |= (def->def_kind == DEF_PROGRAM);

  if (extend && !foundprogram)
  {
    if (!cmd.keep)
       unlink (outfilename);
    return;
  }
  if (nomain)
     write_programs (NULL);
  else
  {
    write_most();
    do_regs (argc, argv);
    write_rest();
    write_programs ("static");
  }
}

static void l_output (char *infile, char *define, int extend, char *outfile)
{
  definition *def;
  int         foundprogram;
  char       *outfilename;

  open_input (infile, define);
  outfilename = extend ? extendfile(infile, outfile) : outfile;
  open_output (infile, outfilename);
  put_headers (infile);

  foundprogram = 0;
  while ((def = get_definition()) != NULL)
        foundprogram |= (def->def_kind == DEF_PROGRAM);

  if (extend && !foundprogram)
  {
    if (!cmd.keep)
       unlink (outfilename);
  }
  else write_stubs();
}

/*
 * Perform registrations for service output 
 */
static void do_regs (int argc, char **argv)
{
  int i;
  for (i = 1; i < argc; i++)
  {
    if (streq(argv[i], "-s"))
    {
      write_register (argv[i + 1]);
      i++;
    }
  }
}

/*
 * Parse command line arguments 
 */
static void parseargs (int argc, char **argv)
{
  int ch;

  cmdname = argv[0];

  while ((ch = getopt(argc, argv, "?avchklms:o:")) != EOF)
     switch (ch)
     {
       case '?': usage (NULL);
                 break;
       case 'a': allfiles = 1;
                 break;
       case 'v': cmd.verbose = 1;
                 break;
       case 'c': cmd.cflag = 1;
                 break;
       case 'h': cmd.hflag = 1;
                 break;
       case 'k': cmd.keep = 1;
                 break;
       case 'l': cmd.lflag = 1;
                 break;
       case 'm': cmd.mflag = 1;
                 break;
       case 's': cmd.sflag = 1;
                 if (!streq(optarg,"udp") && !streq(optarg,"tcp"))
                    usage ("illegal server protocol");
                 break;
       case 'o': cmd.sflag = 1;
                 if (cmd.outfile)
                    usage ("outfile already given");
                 cmd.outfile = optarg;
                 break;
     }

  argc -= optind;
  argv += optind;
  if (argc-- != 1 || (cmd.infile = *argv++) == NULL)
     usage ("no infile");

  infilename = cmd.infile;
}

