/*
 *  Implementation of getrpcxxx() functions for SUN's RPC/XDR
 *  This is part of Sun's RPC/XDR for WatTcp on DOS.
 *
 *  G. Vanem <giva@bgnett.no>
 *
 *  18.aug 1996 - Created
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <rpc/rpc.h>
#include <rpc/pmap_cln.h>
#include <sys/socket.h>
#include <netdb.h>

#include "rpc_mac.h"

struct _rpcent {
        char   *r_name;
        char  **r_aliases;
        long    r_number;    /* must be 32bit for small/large */
        struct _rpcent *next;
      };


#define MAX_RPC_ALIASES  5

static struct _rpcent *rpc0 = NULL;
static char   *rpcFname     = NULL;
static FILE   *rpcFile      = NULL;
static int     rpcClose     = 0;

/*------------------------------------------------------------------*/

void ReadRpcProgFile (const char *fname)
{
  struct _rpcent *r;

  if (!fname || !*fname)
     return;

  rpcFname = strdup (fname);
  setrpcent (1);
  if (!rpcFile)
     return;

  while (1)
  {
    struct _rpcent *r2;
    if ((r = malloc(sizeof(*r))) == NULL)
       break;

    if ((r2 = (struct _rpcent*)getrpcent()) == NULL)
       break;
    *r = *r2;
    r->next = rpc0;
    rpc0    = r;
  }
  atexit (endrpcent);

#if 0  /* test */
  printf ("%s entries:\n",fname);
  for (r = rpc0; r; r = r->next)
  {
    int a;
    printf ("r->r_name = %-15s r->r_number = %5ld  aliases:",
            r->r_name,r->r_number);
    for (a = 0; r->r_aliases && r->r_aliases[a]; a++)
          printf (" %s",r->r_aliases[a]);
    puts ("");
  }
#endif
}

/*
 * return the next (non-commented) line from the rpc-file
 * Format is:
 *   name number [alias ..] {\n | # ..}
 *
 * e.g.
 *   2   rtime
 */
struct rpcent * getrpcent (void)
{
  static struct _rpcent r;
  char  *alias;
  char   buf[200];

  if (!rpcFile)
     return (NULL);

  do
  {
    if (!fgets(buf,sizeof(buf)-1,rpcFile))
       return (NULL);
  }
  while (buf[0] == '#' || buf[0] == ';' || buf[0] == '\n');

  if (rpcClose)
     endrpcent();

  r.r_name    = strdup (strtok(buf, " \t"));
  r.r_number  = atol   (strtok(NULL," \t\n"));
  r.r_aliases = NULL;

  alias = strtok (NULL," \t\n");
  if (alias && *alias != '#' && *alias != ';')
  {
    char **alist = calloc ((1+MAX_RPC_ALIASES) * sizeof(char*), 1);
    int  i = 0;
    do
    {
      if (*alias == '#' || *alias == ';')
         break;
      alist[i++] = strdup (alias);
      if (i == MAX_RPC_ALIASES)
         break;
      alias = strtok (NULL," \t\n");
    }
    while (alias);
    r.r_aliases = alist;
  }
  return (struct rpcent*) &r;
}

/*------------------------------------------------------------------*/

struct rpcent * getrpcbyname (char *name)
{
  struct _rpcent *r;

  for (r = rpc0; r; r = r->next)
  {
    char **alias;

    if (r->r_name && !stricmp(r->r_name,name))
       return (struct rpcent*) r;

    for (alias = r->r_aliases; alias && *alias; alias++)
        if (!strcmp(name,*alias))
           return (struct rpcent*) r;
  }
  return (NULL);
}

struct rpcent *getrpcbynumber (long number) /* was int */
{
  struct _rpcent *r;

  for (r = rpc0; r; r = r->next)
      if (number == r->r_number)
         return (struct rpcent*) r;
  return (NULL); 
}

/*------------------------------------------------------------------*/

void setrpcent (int stayopen)
{
  rpcClose = (stayopen == 0);
  if (!rpcFname)
     return;

  if (!rpcFile)
       rpcFile = fopen (rpcFname,"rt");
  else rewind (rpcFile);
}

/*------------------------------------------------------------------*/

void endrpcent (void)
{
  struct _rpcent *r, *next;

  if (!rpcFile)
     return;  

  free (rpcFname);
  fclose (rpcFile);
  rpcFname = NULL;
  rpcFile  = NULL;

  for (r = rpc0; r; r = next)
  {
    if (r->r_aliases)
    {
      int i;
      for (i = 0; i < MAX_RPC_ALIASES; i++)
          if (r->r_aliases[i])
             free (r->r_aliases[i]);
      free (r->r_aliases);
    }
    next = r->next;
    free (r->r_name);
    free (r);
  }
  rpc0 = NULL;
  rpcClose = 1;
}

/*
 * The following function is
 *
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

int getrpcport (char *host, u_long prognum, u_long versnum, u_long proto)
{
  struct sockaddr_in addr;
  struct hostent    *hp;

  if ((hp = gethostbyname(host)) == NULL)
     return (0);

  memcpy (&addr.sin_addr,hp->h_addr,hp->h_length);
  addr.sin_family = AF_INET;
  addr.sin_port   =  0;
  return pmap_getport (&addr,prognum,versnum,proto);
}
