/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
#if !defined(lint) && defined(SCCSIDS)
static char sccsid[] = "@(#)pmap_clnt.c 1.37 87/08/11 Copyr 1984 Sun Micro";
#endif

/*
 * pmap_clnt.c
 * Client interface to pmap rpc service.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

#include <stdlib.h>
#include <malloc.h>
#include <rpc/rpc.h>
#include <rpc/pmap_pro.h>
#include <rpc/pmap_cln.h>
#include "rpc_mac.h"


static struct timeval timeout    = {  5, 0 };
static struct timeval tottimeout = { 60, 0 };

#ifdef __MSDOS__
  #define CLOSE() sock_close (socket)
  #define FREE()  free (socket)
#else
  #define CLOSE() ((void)0)
  #define FREE()  ((void)0)
#endif

/*
 * Set a mapping between program, version and port.
 * Calls the pmap service remotely to do the mapping.
 */
bool_t pmap_set (u_long program, u_long version, int protocol, int port)
{
  struct sockaddr_in myaddress;
  struct pmap        parms;
  CLIENT *client;
  bool_t rslt;

#ifdef __MSDOS__
  udp_Socket *socket = calloc (sizeof(*socket),1);
  if (!socket)
     return (FALSE);
#else
  int socket = -1;
#endif

  get_myaddress (&myaddress);
  client = clntudp_bufcreate (&myaddress, PMAPPROG, PMAPVERS, timeout,
                              (int*)socket, RPCSMALLMSGSIZE, RPCSMALLMSGSIZE);
  if (!client)
  {
    FREE();
    return (FALSE);
  }

  parms.pm_prog = program;
  parms.pm_vers = version;
  parms.pm_prot = protocol;
  parms.pm_port = port;
  if (CLNT_CALL(client,PMAPPROC_SET,xdr_pmap,&parms,xdr_bool,&rslt,
                       tottimeout) != RPC_SUCCESS)
  {
    clnt_perror (client, "Cannot service");
    CLOSE();
    FREE();
    return (FALSE);
  }

  CLNT_DESTROY (client);

#ifndef __MSDOS__
  close (socket);
#endif
  CLOSE();
  FREE();
  return (rslt);
}

/*
 * Remove the mapping between program,version and port.
 * Calls the pmap service remotely to do the un-mapping.
 */
bool_t pmap_unset (u_long program, u_long version)
{
  struct sockaddr_in myaddress;
  CLIENT *client;
  struct pmap parms;
  bool_t rslt;

#ifdef __MSDOS__
  udp_Socket *socket = calloc (sizeof(*socket),1);
  if (!socket)
     return (FALSE);
#else
  int socket = -1;
#endif

  get_myaddress (&myaddress);
  client = clntudp_bufcreate (&myaddress,PMAPPROG,PMAPVERS,timeout,
                              (int*)socket,RPCSMALLMSGSIZE,RPCSMALLMSGSIZE);
  if (!client)
  {
    FREE();
    return (FALSE);
  }

  parms.pm_prog = program;
  parms.pm_vers = version;
  parms.pm_port = parms.pm_prot = 0;
  CLNT_CALL (client,PMAPPROC_UNSET,xdr_pmap,&parms,xdr_bool,&rslt,tottimeout);
  CLNT_DESTROY (client);

#ifndef __MSDOS__
  close (socket);
#endif
  CLOSE();
  FREE();
  return (rslt);
}
